/***************************************************************************
***
***    Copyright 2007  Hon Hai Precision Ind. Co. Ltd.
***    All Rights Reserved.
***    No portions of this material shall be reproduced in any form without the
***    written permission of Hon Hai Precision Ind. Co. Ltd.
***
***    All information contained in this document is Hon Hai Precision Ind.  
***    Co. Ltd. company private, proprietary, and trade secret property and 
***    are protected by international intellectual property laws and treaties.
***
****************************************************************************/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>

#include <typedefs.h>
#include <bcmutils.h>
#include <sbutils.h>
#include <bcmdevs.h>

#include "wps_led.h"

#undef _DEBUG

static int wps_led_major;
devfs_handle_t wps_leddev_handle;

extern int wps_led_pattern;
extern int wps_led_state;

static int
wps_led_open(struct inode *inode, struct file * file)
{
    MOD_INC_USE_COUNT;
    return 0;
}

static int
wps_led_release(struct inode *inode, struct file * file)
{
    MOD_DEC_USE_COUNT;
    return 0;
}

static int
wps_led_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
    switch (cmd)
    {
        case LED_BLINK_PATTERN:
            wps_led_pattern = (int)arg;
#ifdef _DEBUG
            printk("%s: pattern = 0x%lx\n", __FUNCTION__, wps_led_pattern);
#endif
            break;
            
        case LED_BLINK_NORMAL:
            wps_led_state = 1;
#ifdef _DEBUG
            printk("%s: blink normal\n", __FUNCTION__);
#endif
            break;

        case LED_BLINK_WPS:
            wps_led_state = 2;
#ifdef _DEBUG
            printk("%s: blink WPS\n", __FUNCTION__);
#endif
            break;

        case LED_BLINK_OFF:
            wps_led_state = 0;
#ifdef _DEBUG
            printk("%s: blink OFF\n", __FUNCTION__);
#endif
            break;

        case LED_GET_STATE:
            copy_to_user(arg, &wps_led_state, sizeof(unsigned long));
            break;

        default:
            break;
    }

    return 0;

}
static struct file_operations wps_led_fops = {
    owner:      THIS_MODULE,
    open:       wps_led_open,
    release:    wps_led_release,
    ioctl:      wps_led_ioctl,
    };

static int __init
wps_led_init(void)
{
    if ((wps_led_major = devfs_register_chrdev(0, "wps_led", &wps_led_fops)) < 0)
        return wps_led_major;

    wps_leddev_handle = devfs_register(NULL, "wps_led", DEVFS_FL_DEFAULT,
                                    wps_led_major, 0, S_IFCHR | S_IRUGO | S_IWUGO,
                                    &wps_led_fops, NULL);

    return 0;
}

static void __exit
wps_led_exit(void)
{
    if (wps_leddev_handle != NULL)
        devfs_unregister(wps_leddev_handle);
    wps_leddev_handle = NULL;
    devfs_unregister_chrdev(wps_led_major, "wps_led");
}

module_init(wps_led_init);
module_exit(wps_led_exit);
